/***************************************************************************
                          dlgxmlelement.cpp  -  description
                             -------------------
    begin                : Mit Apr 17 2002
    copyright            : (C) 2002 by The KXMLEditor Team
    email                : lvanek@eanet.cz
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "dlgxmlelement.h"

#include <qlabel.h>
#include <qcombobox.h>
#include <qpushbutton.h>

#include <klineedit.h>
#include <kdebug.h>

DlgXMLElement::DlgXMLElement( QWidget * pParent, const char * pszName, bool fModal, WFlags fl )
	: DlgXMLElementBase( pParent, pszName, fModal, fl )
{
	connect( m_pEditNsURI, SIGNAL(textChanged(const QString&)), this, SLOT(slotNsURIChanged(const QString&)) );
	connect( m_pEditName, SIGNAL(textChanged(const QString&)), this, SLOT(slotNameChanged(const QString&)) );
}

void DlgXMLElement::clearDialog()
{
	m_pEditNsURI->clear();
	m_pEditNsURI->setEnabled(true);
	m_pEditNsURI->show();
	m_pLblNsURI->setEnabled(true);
	m_pLblNsURI->show();

	m_pEditPrefix->clear();
	m_pEditPrefix->setDisabled(true);
	m_pEditPrefix->show();
	m_pLblPrefix->setEnabled(true);
	m_pLblPrefix->show();

	m_pEditName->clear();

	m_pComboInsert->setCurrentItem(0);
}

void DlgXMLElement::fillDialog( const QDomElement & domElement )
{
	m_pEditNsURI->setDisabled(true);

	if ( domElement.namespaceURI().isNull() )
	{
		m_pEditNsURI->hide();
		m_pLblNsURI->hide();
		m_pLblNsURI->setDisabled(true);

		m_pEditPrefix->setDisabled(true);
		m_pEditPrefix->hide();
		m_pLblPrefix->hide();
		m_pLblPrefix->setDisabled(true);
	}
	else
	{
		m_pEditNsURI->setText( domElement.namespaceURI() );
		m_pEditNsURI->show();
		m_pLblNsURI->show();
		m_pLblNsURI->setEnabled(true);

		m_pEditPrefix->setText( domElement.prefix() );
		m_pEditPrefix->setEnabled(true);
		m_pEditPrefix->show();
		m_pLblPrefix->show();
		m_pLblPrefix->setEnabled(true);
	}

	m_pComboInsert->hide();
	m_pComboInsert->setDisabled(true);
	m_pLblInsert->hide();
	m_pLblInsert->setDisabled(true);

	m_pEditName->setText( domElement.tagName() );
}

void DlgXMLElement::fillElement( QDomElement & domElement )
{
	if ( ! domElement.namespaceURI().isNull() )
		domElement.setPrefix( m_pEditPrefix->text() );

	domElement.setTagName( m_pEditName->text() );
}

int DlgXMLElement::exec( QDomElement & domElement )
{
	fillDialog( domElement );

	int iReturn = exec();
	if ( iReturn == Accepted )
		fillElement( domElement );

	return iReturn;
}

int DlgXMLElement::exec( QDomDocument & domDoc, QDomElement & domNewElement )
{
	if ( domDoc.isNull() )
	{
		kdError() << "DlgXMLElement::exec the given XML document object is empty." << endl;
		return Rejected;
	}
	if ( ! domDoc.documentElement().isNull() ) // if the document already
	{                                          // has a root element
		kdError() << "DlgXMLElement::exec the given XML document already has a root element." << endl;
		return Rejected;
	}

	m_pComboInsert->hide();
	m_pComboInsert->setDisabled(true);
	m_pLblInsert->hide();
	m_pLblInsert->setDisabled(true);

	clearDialog();

	int iReturn = exec();
	if ( iReturn == Accepted )
	{
		if ( m_pEditNsURI->text().isEmpty() )
			domNewElement = domDoc.createElement( m_pEditName->text() );
		else
			domNewElement = domDoc.createElementNS( m_pEditNsURI->text(),
			                                        m_pEditPrefix->text() + ":" + m_pEditName->text() );

		domDoc.appendChild( domNewElement );
	}

	return iReturn;
}

int DlgXMLElement::exec( QDomElement & domParentElement, QDomElement & domNewElement )
{
	if ( domParentElement.isNull() )
	{
		kdError() << "DlgXMLElement::exec the given parent XML element object is empty." << endl;
		return Rejected;
	}

	clearDialog();

	int iReturn = exec();
	if ( iReturn == Accepted )
	{
		if ( m_pEditNsURI->text().isEmpty() )
			domNewElement = domParentElement.ownerDocument().createElement( m_pEditName->text() );
		else
			domNewElement = domParentElement.ownerDocument().createElementNS( m_pEditNsURI->text(),
			                                                                  m_pEditPrefix->text() + ":" + m_pEditName->text() );

		if ( m_pComboInsert->currentItem() == 0 )
		{                                         // insert as first child
			QDomNode domFirstChildNode = domParentElement.firstChild();
			if ( domFirstChildNode.isNull() )
				domParentElement.appendChild( domNewElement ); // no childs yet -> simply append
			else
				domParentElement.insertBefore( domNewElement, domFirstChildNode );
		}
		else                                      // insert as last child
			domParentElement.appendChild( domNewElement );
	}

	return iReturn;
}

int DlgXMLElement::exec()
{
	if ( m_pEditName->text().isEmpty() )
		m_pBtnOK->setEnabled(false);
	else
		m_pBtnOK->setEnabled(true);
		
	m_pEditName->setFocus();
	m_pBtnOK->setDefault(true);

	return DlgXMLElementBase::exec();
}

void DlgXMLElement::slotNsURIChanged( const QString & szNewNsURI )
{
	if ( szNewNsURI.isEmpty() )
		m_pEditPrefix->setEnabled(false);
	else
		m_pEditPrefix->setEnabled(true);
}

void DlgXMLElement::slotNameChanged( const QString & szNewName )
{
	if ( szNewName.isEmpty() )
		m_pBtnOK->setEnabled(false);
	else
		m_pBtnOK->setEnabled(true);
}
