/***************************************************************************
                          dlgxmlelement.h  -  description
                             -------------------
    begin                : Mit Apr 17 2002
    copyright            : (C) 2002 by The KXMLEditor Team
    email                : lvanek@eanet.cz
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef DLGXMLELEMENT_H
#define DLGXMLELEMENT_H

#include "dlgxmlelementbase.h"
#include "qdom.h"

class DlgXMLElement : public DlgXMLElementBase
{
		Q_OBJECT

	public:
		DlgXMLElement( QWidget * pParent = 0, const char * pszName = 0, bool fModal = true, WFlags fl = 0 );

	public slots:
		/**
		 * To be used for editing the given XML element, therefore
		 * hides the "Where to insert" widgets.
		 * Updates the given element with the data entered by the user (if OK was pressed).
		 *
		 * @return see @ref QDialog::exec
		 */
		int exec( QDomElement & domElement );
		/**
		 * To be used for creating a new XML element as root element
		 * (therefore hides the "Where to insert" widgets).
		 * If OK was pressed, a new element is created with the entered
		 * data and gets set as the given XML documents root element.
		 * The new element can be accessed by domNewElement.
		 *
		 * @return see @ref QDialog::exec
		 */
		int exec( QDomDocument & domDoc, QDomElement & domNewElement );
		/**
		 * To be used for creating a new XML element as a child element
		 * of the given parent element.
		 * If OK was pressed, a new element is created with the entered
		 * data and gets set as a child element of the given parent element.
		 * The new element can be accessed by domNewElement.
		 *
		 * @return see @ref QDialog::exec
		 */
		int exec( QDomElement & domParentElement, QDomElement & domNewElement );

		/**
		 * Clears the dialogs widgets (removes their content).
		 */
		void clearDialog();
		/**
		 * Fills the dialogs widgets with the data from the given XML element.
		 */
		void fillDialog( const QDomElement & domElement );
		/**
		 * Updates the given element with the data entered in the dialogs widgets.
		 */
		void fillElement( QDomElement & domElement );

	protected slots:
		/**
		 * Don't use this function directly, it's for internal use only.
		 * Use one of the other @ref DlgXMLElement::exec functions instead.
		 */
		int exec();
		/**
		 * Called, when the namespace URI in the edit line @ref DlgXMLElementBase::m_pEditNsURI
		 * is changed.
		 * Disables the prefix edit line (@ref DlgXMLElementBase::m_pEditPrefix),
		 * if the given string is empty.
		 */
		void slotNsURIChanged( const QString & szNewNsURI );
		/**
		 * Called, when the name in the edit line @ref DlgXMLElementBase::m_pEditName
		 * is changed.
		 * Disables the OK button, if the given string is empty.
		 */
		void slotNameChanged( const QString & szNewName );

};

#endif
